import os
import sys
import tarfile
from urllib.request import urlretrieve
import numpy as np
import tensorflow as tf
import dataset_utils
from constants import STL10_DATADIR, STL10_TF_DATADIR

# image shape
HEIGHT = 96
WIDTH = 96
DEPTH = 3

# size of a single image in bytes
SIZE = HEIGHT * WIDTH * DEPTH

# url of the binary data
DATA_URL = "http://ai.stanford.edu/~acoates/stl10/stl10_binary.tar.gz"

# path to the binary train file with image data
DATA_PATH = os.path.join(STL10_DATADIR, "stl10_binary/train_X.bin")

# path to the binary train file with labels
LABEL_PATH = os.path.join(STL10_DATADIR, "stl10_binary/train_y.bin")


def read_labels(path_to_labels):
    """
    :param path_to_labels: path to the binary file containing labels from the STL-10 dataset
    :return: an array containing the labels
    """
    with open(path_to_labels, "rb") as f:
        return np.fromfile(f, dtype=np.uint8)


def read_all_images(path_to_data):
    """
    :param path_to_data: the file containing the binary images from the STL-10 dataset
    :return: an array containing all the images
    """

    with open(path_to_data, "rb") as f:
        # read whole file in uint8 chunks
        everything = np.fromfile(f, dtype=np.uint8)

        # We force the data into 3x96x96 chunks, since the
        # images are stored in "column-major order", meaning
        # that "the first 96*96 values are the red channel,
        # the next 96*96 are green, and the last are blue."
        # The -1 is since the size of the pictures depends
        # on the input file, and this way numpy determines
        # the size on its own.

        images = np.reshape(everything, (-1, 3, 96, 96))

        # Now transpose the images into a standard image format
        # readable by, for example, matplotlib.imshow
        # You might want to comment this line or reverse the shuffle
        # if you will use a learning algorithm like CNN, since they like
        # their channels separated.
        images = np.transpose(images, (0, 3, 2, 1))
        return images


def read_single_image(image_file):
    """
    CAREFUL! - this method uses a file as input instead of the path - so the
    position of the reader will be remembered outside of context of this method.
    :param image_file: the open file containing the images
    :return: a single image
    """
    # read a single image, count determines the number of uint8's to read
    image = np.fromfile(image_file, dtype=np.uint8, count=SIZE)
    # force into image matrix
    image = np.reshape(image, (3, 96, 96))
    # transpose to standard format
    # You might want to comment this line or reverse the shuffle
    # if you will use a learning algorithm like CNN, since they like
    # their channels separated.
    image = np.transpose(image, (2, 1, 0))
    return image


def _add_to_tfrecord(
    data_filename, tfrecord_writer, label_filename=None, subset=None, test_fold=False
):
    """Loads data from the stl10 files and writes to a TFRecord.
    Args:
      data_filename: The filename of the stl10 file.
      tfrecord_writer: The TFRecord writer to use for writing.
    Returns:
      The new offset.
    """

    images = read_all_images(data_filename)
    labels = read_labels(label_filename) if label_filename else None
    if subset:
        if test_fold:
            images = images[subset]
            labels = labels[subset]
        else:
            images = np.delete(images, subset, axis=0)
            labels = np.delete(labels, subset, axis=0)

    num_images = images.shape[0]

    with tf.Graph().as_default():
        coder = dataset_utils.ImageCoder()

        with tf.Session(""):
            for j in range(num_images):
                sys.stdout.write(
                    "\r>> Reading file [%s] image %d/%d"
                    % (data_filename, j + 1, num_images)
                )
                sys.stdout.flush()

                # Get image, edge-map and cartooned image
                image = np.squeeze(images[j])
                label = labels[j] - 1 if label_filename else -1
                # Build example
                img_str = coder.encode_jpeg(image)

                example = dataset_utils.image_example(image, img_str, int(label))
                tfrecord_writer.write(example.SerializeToString())

    return num_images


def download_and_extract():
    """
    Downloads the stl-10 dataset
    """
    dest_directory = 'data'
    if not os.path.exists(dest_directory):
        os.makedirs(dest_directory)
    filename = DATA_URL.split("/")[-1]
    filepath = os.path.join(dest_directory, filename)
    if not os.path.exists(filepath):

        def _progress(count, block_size, total_size):
            sys.stdout.write(
                "\rDownloading %s %.2f%%"
                % (filename, float(count * block_size) / float(total_size) * 100.0)
            )
            sys.stdout.flush()

        filepath, _ = urlretrieve(DATA_URL, filepath, reporthook=_progress)
        print("Downloaded", filename)
        tarfile.open(filepath, "r:gz").extractall(dest_directory)


def run():
    # download data if needed
    download_and_extract()

    # check if the whole dataset is read correctly
    images = read_all_images(DATA_PATH)
    print(f"Found images: {images.shape}")
    labels = read_labels(LABEL_PATH)
    print(f"Found labels: {labels.shape}")

    # Make dataset folder if necessary
    if not os.path.exists(STL10_TF_DATADIR):
        os.mkdir(STL10_TF_DATADIR)

    # Make unlabeled dataset
    unlabeled_train_filename = os.path.join(
        STL10_DATADIR, "stl10_binary/unlabeled_X.bin"
    )
    unlabeled_train_tf_file = os.path.join(
        STL10_TF_DATADIR, "stl10_train_unlabeled.tfrecord"
    )
    with tf.python_io.TFRecordWriter(unlabeled_train_tf_file) as tfrecord_writer:
        num_written = _add_to_tfrecord(unlabeled_train_filename, tfrecord_writer)
    print(f"Wrote {num_written} images to {unlabeled_train_tf_file}")

    num_written = _extracted_from_run_27(
        "stl10_binary/train_X.bin",
        "stl10_train.tfrecord",
        "stl10_binary/train_y.bin",
    )
    num_written = _extracted_from_run_27(
        "stl10_binary/test_X.bin",
        "stl10_test.tfrecord",
        "stl10_binary/test_y.bin",
    )


# TODO Rename this here and in `run`
def _extracted_from_run_27(arg0, arg1, arg2):
    # Make record with the full clean dataset
    labeled_train_filename = os.path.join(STL10_DATADIR, arg0)
    labeled_train_tf_file = os.path.join(STL10_TF_DATADIR, arg1)
    label_file_train = os.path.join(STL10_DATADIR, arg2)
    with tf.python_io.TFRecordWriter(labeled_train_tf_file) as tfrecord_writer:
        result = _add_to_tfrecord(
            labeled_train_filename,
            tfrecord_writer,
            label_filename=label_file_train,
        )
    print(f"Wrote {result} images to {labeled_train_tf_file}")

    return result


if __name__ == "__main__":
    run()



# Copyright 2016 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Contains utilities for downloading and converting datasets."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import sys
import tarfile

import numpy as np
import tensorflow as tf

LABELS_FILENAME = "labels.txt"


def _bytes_feature(value):
    """Returns a bytes_list from a string / byte."""
    return tf.train.Feature(bytes_list=tf.train.BytesList(value=[value]))


def _float_feature(value):
    """Returns a float_list from a float / double."""
    return tf.train.Feature(float_list=tf.train.FloatList(value=[value]))


def _int64_feature(values):
    """Returns a TF-Feature of int64s.
    Args:
      values: A scalar or list of values.
    Returns:
      a TF-Feature.
    """
    if not isinstance(values, (tuple, list)):
        values = [values]
    return tf.train.Feature(int64_list=tf.train.Int64List(value=values))


def image_example(image, img_str, label):
    image_shape = image.shape

    feature = {
        "height": _int64_feature(image_shape[0]),
        "width": _int64_feature(image_shape[1]),
        "depth": _int64_feature(image_shape[2]),
        "label": _int64_feature(label),
        "image_raw": _bytes_feature(img_str),
    }

    return tf.train.Example(features=tf.train.Features(feature=feature))


def image_example_celeba(image, img_str, label_att, label_id):
    image_shape = image.shape

    feature = {
        "height": _int64_feature(image_shape[0]),
        "width": _int64_feature(image_shape[1]),
        "depth": _int64_feature(image_shape[2]),
        "label": _int64_feature(label_att),
        "id": _int64_feature(label_id),
        "image_raw": _bytes_feature(img_str),
    }

    return tf.train.Example(features=tf.train.Features(feature=feature))


class ImageCoder(object):
    """Helper class that provides TensorFlow image coding utilities."""

    def __init__(self):
        # Create a single Session to run all image coding calls.
        self._sess = tf.Session()

        # Initializes function that converts PNG to JPEG data.
        self._png_data = tf.placeholder(dtype=tf.string)
        image = tf.image.decode_png(self._png_data, channels=3)
        self._png_to_jpeg = tf.image.encode_jpeg(image, format="rgb", quality=100)

        # Initializes function that converts CMYK JPEG data to RGB JPEG data.
        self._cmyk_data = tf.placeholder(dtype=tf.string)
        image = tf.image.decode_jpeg(self._cmyk_data, channels=0)
        self._cmyk_to_rgb = tf.image.encode_jpeg(image, format="rgb", quality=100)

        # Initializes function that decodes RGB JPEG data.
        self._decode_jpeg_data = tf.placeholder(dtype=tf.string)
        self._decode_jpeg = tf.image.decode_jpeg(self._decode_jpeg_data, channels=3)

        # Initializes function that encodes RGB JPEG data.
        self._encode_image_data = tf.placeholder(dtype=tf.uint8)
        self._encode_jpeg = tf.image.encode_jpeg(self._encode_image_data)

    def png_to_jpeg(self, image_data):
        return self._sess.run(self._png_to_jpeg, feed_dict={self._png_data: image_data})

    def cmyk_to_rgb(self, image_data):
        return self._sess.run(
            self._cmyk_to_rgb, feed_dict={self._cmyk_data: image_data}
        )

    def decode_jpeg(self, image_data):
        image = self._sess.run(
            self._decode_jpeg, feed_dict={self._decode_jpeg_data: image_data}
        )
        assert len(image.shape) == 3
        assert image.shape[2] == 3
        return image

    def encode_jpeg(self, image_data):
        image_data = image_data.astype(dtype=np.uint8)
        return self._sess.run(
            self._encode_jpeg, feed_dict={self._encode_image_data: image_data}
        )


def write_label_file(labels_to_class_names, dataset_dir, filename=LABELS_FILENAME):
    """Writes a file with the list of class names.
    Args:
      labels_to_class_names: A map of (integer) labels to class names.
      dataset_dir: The directory in which the labels file should be written.
      filename: The filename where the class names are written.
    """
    labels_filename = os.path.join(dataset_dir, filename)
    with tf.gfile.Open(labels_filename, "w") as f:
        for label in labels_to_class_names:
            class_name = labels_to_class_names[label]
            f.write("%d:%s\n" % (label, class_name))



import os

# Directories for data and logs
DATA_DIR = 'stl10'
LOG_DIR = os.path.join(DATA_DIR, 'Logs')
TFREC_DIR = os.path.join(DATA_DIR, 'TF_Records')
DATASETS_DIR = os.path.join(DATA_DIR, 'Datasets')

# TFRecord directories
STL10_TF_DATADIR = os.path.join(TFREC_DIR, 'STL10_TFRecords/')

# Source directories for datasets
STL10_DATADIR = os.path.join(DATASETS_DIR, 'STL10/')


import os
import tensorflow as tf
from constants import STL10_TF_DATADIR

slim = tf.contrib.slim


class STL10:

    SPLITS_TO_SIZES = {
        "train_unlabeled": 100000,
        "train": 5000,
        "test": 8000,
        "train_fold_0": 4000,
        "train_fold_1": 4000,
        "train_fold_2": 4000,
        "train_fold_3": 4000,
        "train_fold_4": 4000,
        "train_fold_5": 4000,
        "train_fold_6": 4000,
        "train_fold_7": 4000,
        "train_fold_8": 4000,
        "train_fold_9": 4000,
        "test_fold_0": 1000,
        "test_fold_1": 1000,
        "test_fold_2": 1000,
        "test_fold_3": 1000,
        "test_fold_4": 1000,
        "test_fold_5": 1000,
        "test_fold_6": 1000,
        "test_fold_7": 1000,
        "test_fold_8": 1000,
        "test_fold_9": 1000,
    }

    def __init__(self, split_name="train_unlabeled"):
        self.split_name = split_name
        self.reader = tf.TFRecordReader
        self.label_offset = 0
        self.is_multilabel = False
        self.data_dir = STL10_TF_DATADIR
        self.file_pattern = "stl10_%s.tfrecord"
        self.num_classes = 10
        self.name = "STL10"
        self.num_samples = self.SPLITS_TO_SIZES[split_name]

    def feature_description(self):
        return {
            "height": tf.io.FixedLenFeature([], tf.int64),
            "width": tf.io.FixedLenFeature([], tf.int64),
            "depth": tf.io.FixedLenFeature([], tf.int64),
            "label": tf.io.FixedLenFeature([], tf.int64),
            "image_raw": tf.io.FixedLenFeature([], tf.string),
        }

    def get_items_to_handlers(self):
        return {
            "image": slim.tfexample_decoder.Image(
                "image/encoded", "image/format", shape=[96, 96, 3], channels=3
            ),
            "height": slim.tfexample_decoder.Tensor("image/height"),
            "width": slim.tfexample_decoder.Tensor("image/width"),
            "label": slim.tfexample_decoder.Tensor("image/class/label"),
        }

    def get_trainset_labelled(self):
        return self.get_split("train")

    def get_trainset_unlabelled(self):
        return self.get_split("train_unlabeled")

    def get_testset(self):
        return self.get_split("test")

    def get_train_fold_id(self, fold_idx):
        return f"train_fold_{fold_idx}"

    def get_test_fold_id(self, fold_idx):
        return f"test_fold_{fold_idx}"

    def get_num_train_labelled(self):
        return self.SPLITS_TO_SIZES["train"]

    def get_num_train_unlabelled(self):
        return self.SPLITS_TO_SIZES["train_unlabeled"]

    def get_num_test(self):
        return self.SPLITS_TO_SIZES["test"]

    def get_split_size(self, split_name):
        return self.SPLITS_TO_SIZES[split_name]

    def format_labels(self, labels):
        return slim.one_hot_encoding(labels, self.num_classes)

    def get_split(self, split_name, data_dir=None):
        """Gets a dataset tuple with instructions for reading ImageNet.
        Args:
          split_name: A train/eval split name.
          data_dir: The base directory of the dataset sources.
        Returns:
          A `Dataset` namedtuple.
        Raises:
          ValueError: if `split_name` is not a valid train/eval split.
        """
        if split_name not in self.SPLITS_TO_SIZES:
            raise ValueError(f"split name {split_name} was not recognized.")

        if not data_dir:
            data_dir = self.data_dir

        tf_record_pattern = os.path.join(data_dir, self.file_pattern % split_name)
        data_files = tf.io.gfile.glob(tf_record_pattern)
        if not data_files:
            print(f"No files found for dataset at {data_dir}")
        print("data_files: {}".format(data_files))

        raw_dataset = tf.data.TFRecordDataset(data_files)

        # Build the decoder
        feature_description = self.feature_description()

        def _parse_function(example_proto):
            return tf.io.parse_single_example(example_proto, feature_description)

        parsed_dataset = raw_dataset.map(_parse_function)

        return parsed_dataset

    def get_dataset(self):
        return self.get_split(self.split_name)



from setuptools import setup, find_packages
import codecs
import os


VERSION = '0.0.4'
DESCRIPTION = 'Easy to use STL10'
LONG_DESCRIPTION = 'build stl-10 dataset'

# Setting up
setup(
    name="STL10",
    version=VERSION,
    author="Shwky Mohamed",
    author_email="shwkym54@gmail.com",
    description=DESCRIPTION,
    # long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['opencv-python', 'tensorflow', 'numpy'],
    keywords=['python', 'ML', 'dataset', 'stl-10', 'STL10'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)